"""Build fuzzers idempotently in a given folder."""

import argparse
from pathlib import Path
import subprocess


def main():
    """Build fuzzers idempotently in a given folder."""
    parser = argparse.ArgumentParser()
    parser.add_argument("folder")
    args = parser.parse_args()
    folder = Path(args.folder)
    if not folder.exists():
        print(f"Cloning google/oss-fuzz into: {folder}")
        folder.mkdir(parents=True)
        subprocess.check_call(
            [
                "git",
                "clone",
                "--single-branch",
                "https://github.com/google/oss-fuzz",
                str(folder),
            ]
        )
    else:
        print(f"Using google/oss-fuzz in: {folder}")
    if not (folder / "build").exists():
        print(f"Building fuzzers in: {folder / 'build'}")
        subprocess.check_call(
            [
                "python",
                str(folder / "infra" / "helper.py"),
                "build_fuzzers",
                "markdown-it-py",
            ]
        )
    else:
        print(f"Using existing fuzzers in: {folder / 'build'}")


if __name__ == "__main__":
    main()



import warnings

from markdown_it.token import Token


def test_token():
    token = Token("name", "tag", 0)
    assert token.as_dict() == {
        "type": "name",
        "tag": "tag",
        "nesting": 0,
        "attrs": None,
        "map": None,
        "level": 0,
        "children": None,
        "content": "",
        "markup": "",
        "info": "",
        "meta": {},
        "block": False,
        "hidden": False,
    }
    token.attrSet("a", "b")
    assert token.attrGet("a") == "b"
    token.attrJoin("a", "c")
    assert token.attrGet("a") == "b c"
    token.attrPush(("x", "y"))
    assert token.attrGet("x") == "y"
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        assert token.attrIndex("a") == 0
        assert token.attrIndex("x") == 1
        assert token.attrIndex("j") == -1


def test_serialization():
    token = Token("name", "tag", 0, children=[Token("other", "tag2", 0)])
    assert token == Token.from_dict(token.as_dict())



import pytest

from markdown_it import MarkdownIt


@pytest.mark.parametrize(
    "input,expected",
    [
        ("#", "<h1></h1>\n"),
        ("###", "<h3></h3>\n"),
        ("` `", "<p><code> </code></p>\n"),
        ("``````", "<pre><code></code></pre>\n"),
        ("-", "<ul>\n<li></li>\n</ul>\n"),
        ("1.", "<ol>\n<li></li>\n</ol>\n"),
        (">", "<blockquote></blockquote>\n"),
        ("---", "<hr />\n"),
        ("<h1></h1>", "<h1></h1>"),
        ("p", "<p>p</p>\n"),
        ("[reference]: /url", ""),
        ("    indented code block", "<pre><code>indented code block\n</code></pre>\n"),
        ("> test\n>", "<blockquote>\n<p>test</p>\n</blockquote>\n"),
    ],
)
def test_no_end_newline(input, expected):
    md = MarkdownIt()
    text = md.render(input)
    assert text == expected



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
from glob import glob
import os

# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = "markdown-it-py"
copyright = "2020, executable book project"
author = "executable book project"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.intersphinx",
    "myst_parser",
    "sphinx_copybutton",
    "sphinx_design",
    "jupyter_sphinx",
]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

nitpicky = True
nitpick_ignore_regex = [
    ("py:.*", name)
    for name in (
        ".*_ItemTV",
        ".*_NodeType",
        ".*Literal.*",
        ".*_Result",
        ".*_State",
        "EnvType",
        "Path",
        "Ellipsis",
        "NotRequired",
    )
]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_title = "markdown-it-py"
html_logo = html_favicon = "_static/markdown-it-py.svg"
html_theme = "sphinx_book_theme"
html_theme_options = {
    "home_page_in_toc": True,
    "use_repository_button": True,
    "use_edit_page_button": True,
    "repository_url": "https://github.com/executablebooks/markdown-it-py",
    "repository_branch": "master",
    "path_to_docs": "docs",
}
html_static_path = ["_static"]
html_css_files = ["custom.css"]

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]


intersphinx_mapping = {
    "python": ("https://docs.python.org/3.10", None),
    "mdit-py-plugins": ("https://mdit-py-plugins.readthedocs.io/en/latest/", None),
}


def run_apidoc(app):
    """generate apidoc

    See: https://github.com/rtfd/readthedocs.org/issues/1139
    """
    import os
    import shutil

    import sphinx
    from sphinx.ext import apidoc

    logger = sphinx.util.logging.getLogger(__name__)
    logger.info("running apidoc")
    # get correct paths
    this_folder = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
    api_folder = os.path.join(this_folder, "api")
    module_path = os.path.normpath(os.path.join(this_folder, "../"))
    ignore_paths = ["../scripts", "../conftest.py", "../tests", "../benchmarking"]
    ignore_paths = [
        os.path.normpath(os.path.join(this_folder, p)) for p in ignore_paths
    ]
    # functions from these modules are all imported in the __init__.py with __all__
    for rule in ("block", "core", "inline"):
        for path in glob(
            os.path.normpath(
                os.path.join(this_folder, f"../markdown_it/rules_{rule}/*.py")
            )
        ):
            if os.path.basename(path) not in ("__init__.py", f"state_{rule}.py"):
                ignore_paths.append(path)

    if os.path.exists(api_folder):
        shutil.rmtree(api_folder)
    os.mkdir(api_folder)

    argv = ["-M", "--separate", "-o", api_folder, module_path, *ignore_paths]

    apidoc.OPTIONS.append("ignore-module-all")
    apidoc.main(argv)

    # we don't use this
    if os.path.exists(os.path.join(api_folder, "modules.rst")):
        os.remove(os.path.join(api_folder, "modules.rst"))


def setup(app):
    """Add functions to the Sphinx setup."""
    if os.environ.get("SKIP_APIDOC", None) is None:
        app.connect("builder-inited", run_apidoc)



from markdown_it import MarkdownIt


def test_ref_definitions():
    md = MarkdownIt()
    src = "[a]: abc\n\n[b]: xyz\n\n[b]: ijk"
    env = {}  # type: ignore
    tokens = md.parse(src, env)
    assert tokens == []
    assert env == {
        "references": {
            "A": {"title": "", "href": "abc", "map": [0, 1]},
            "B": {"title": "", "href": "xyz", "map": [2, 3]},
        },
        "duplicate_refs": [{"href": "ijk", "label": "B", "map": [4, 5], "title": ""}],
    }


def test_use_existing_env(data_regression):
    md = MarkdownIt()
    src = "[a]\n\n[c]: ijk"
    env = {
        "references": {
            "A": {"title": "", "href": "abc", "map": [0, 1]},
            "B": {"title": "", "href": "xyz", "map": [2, 3]},
        }
    }
    tokens = md.parse(src, env)
    data_regression.check([token.as_dict() for token in tokens])
    assert env == {
        "references": {
            "A": {"title": "", "href": "abc", "map": [0, 1]},
            "B": {"title": "", "href": "xyz", "map": [2, 3]},
            "C": {"title": "", "href": "ijk", "map": [2, 3]},
        }
    }


def test_store_labels(data_regression):
    md = MarkdownIt()
    md.options["store_labels"] = True
    src = "[a]\n\n![a]\n\n[a]: ijk"
    tokens = md.parse(src)
    data_regression.check([token.as_dict() for token in tokens])


def test_inline_definitions(data_regression):
    md = MarkdownIt()
    md.options["inline_definitions"] = True
    src = '[a]: url "title"\n- [a]: url "title"'
    tokens = md.parse(src)
    data_regression.check([token.as_dict() for token in tokens])



from pathlib import Path
from shutil import which

import pytest


@pytest.fixture
def spec_text():
    return Path(__file__).parent.joinpath("samples", "spec.md").read_text()


@pytest.mark.benchmark(group="packages")
def test_markdown_it_py(benchmark, spec_text):
    import markdown_it

    parser = markdown_it.MarkdownIt("commonmark")
    benchmark.extra_info["version"] = markdown_it.__version__
    benchmark(parser.render, spec_text)


@pytest.mark.benchmark(group="packages")
def test_markdown_it_pyrs(benchmark, spec_text):
    import markdown_it_pyrs

    parser = markdown_it_pyrs.MarkdownIt("commonmark")
    benchmark.extra_info["version"] = markdown_it_pyrs.__version__
    benchmark(parser.render, spec_text)


@pytest.mark.benchmark(group="packages")
def test_mistune(benchmark, spec_text):
    import mistune

    benchmark.extra_info["version"] = mistune.__version__
    benchmark(mistune.markdown, spec_text)


@pytest.mark.benchmark(group="packages")
def test_commonmark_py(benchmark, spec_text):
    import commonmark

    benchmark.extra_info["version"] = "0.9.1"
    benchmark(commonmark.commonmark, spec_text)


@pytest.mark.benchmark(group="packages")
def test_pymarkdown(benchmark, spec_text):
    import markdown

    benchmark.extra_info["version"] = markdown.__version__
    benchmark(markdown.markdown, spec_text)


@pytest.mark.benchmark(group="packages")
def test_pymarkdown_extra(benchmark, spec_text):
    import markdown

    benchmark.extra_info["version"] = markdown.__version__
    benchmark(markdown.markdown, spec_text, extensions=["extra"])


@pytest.mark.benchmark(group="packages")
def test_mistletoe(benchmark, spec_text):
    import mistletoe

    benchmark.extra_info["version"] = mistletoe.__version__
    benchmark(mistletoe.markdown, spec_text)


@pytest.mark.skipif(which("pandoc") is None, reason="pandoc executable not found")
@pytest.mark.benchmark(group="packages")
def test_panflute(benchmark, spec_text):
    import panflute

    benchmark.extra_info["version"] = panflute.__version__
    benchmark(
        panflute.convert_text, spec_text, input_format="markdown", output_format="html"
    )



"""In this module tests are run against the full test set,
provided by https://github.com/commonmark/CommonMark.git.
"""

import json
from pathlib import Path

import pytest

from markdown_it import MarkdownIt

SPEC_INPUT = Path(__file__).parent.joinpath("spec.md")
TESTS_INPUT = Path(__file__).parent.joinpath("commonmark.json")


def test_file(file_regression):
    md = MarkdownIt("commonmark")
    file_regression.check(md.render(SPEC_INPUT.read_text()), extension=".html")


@pytest.mark.parametrize("entry", json.loads(TESTS_INPUT.read_text()))
def test_spec(entry):
    md = MarkdownIt("commonmark")
    output = md.render(entry["markdown"])
    expected = entry["html"]

    if entry["example"] == 596:
        # this doesn't have any bearing on the output
        output = output.replace("mailto", "MAILTO")
    if entry["example"] in [218, 239, 240]:
        # this doesn't have any bearing on the output
        output = output.replace(
            "<blockquote></blockquote>", "<blockquote>\n</blockquote>"
        )

    assert output == expected



from pathlib import Path

import pytest

from markdown_it import MarkdownIt
from markdown_it.utils import read_fixture_file

FIXTURE_PATH = Path(__file__).parent.joinpath("fixtures")


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("linkify.md")),
)
def test_linkify(line, title, input, expected):
    md = MarkdownIt().enable("linkify")
    md.options["linkify"] = True
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()

    # if not install linkify-it-py
    md.linkify = None
    with pytest.raises(ModuleNotFoundError):
        md.render(input)


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("smartquotes.md")),
)
def test_smartquotes(line, title, input, expected):
    md = MarkdownIt().enable("replacements").enable("smartquotes")
    md.options["typographer"] = True
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("typographer.md")),
)
def test_typographer(line, title, input, expected):
    md = MarkdownIt().enable("replacements")
    md.options["typographer"] = True
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected", read_fixture_file(FIXTURE_PATH.joinpath("tables.md"))
)
def test_table(line, title, input, expected):
    md = MarkdownIt().enable("table")
    text = md.render(input)
    try:
        assert text.rstrip() == expected.rstrip()
    except AssertionError:
        print(text)
        raise


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("commonmark_extras.md")),
)
def test_commonmark_extras(line, title, input, expected):
    md = MarkdownIt("commonmark")
    md.options["langPrefix"] = ""
    text = md.render(input)
    if text.rstrip() != expected.rstrip():
        print(text)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("normalize.md")),
)
def test_normalize_url(line, title, input, expected):
    md = MarkdownIt("commonmark")
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected", read_fixture_file(FIXTURE_PATH.joinpath("fatal.md"))
)
def test_fatal(line, title, input, expected):
    md = MarkdownIt("commonmark").enable("replacements")
    md.options["typographer"] = True
    text = md.render(input)
    if text.rstrip() != expected.rstrip():
        print(text)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("strikethrough.md")),
)
def test_strikethrough(line, title, input, expected):
    md = MarkdownIt().enable("strikethrough")
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("strikethrough_single_tilde.md")),
)
def test_strikethrough_single_tilde(line, title, input, expected):
    md = MarkdownIt("gfm-like2")
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("disable_code_block.md")),
)
def test_disable_code_block(line, title, input, expected):
    md = MarkdownIt().enable("table").disable("code")
    text = md.render(input)
    print(text.rstrip())
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("tasklists.md")),
)
def test_tasklists(line, title, input, expected):
    md = MarkdownIt("gfm-like2")
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("alerts.md")),
)
def test_alerts(line, title, input, expected):
    md = MarkdownIt("gfm-like2")
    text = md.render(input)
    assert text.rstrip() == expected.rstrip()


@pytest.mark.parametrize(
    "line,title,input,expected",
    read_fixture_file(FIXTURE_PATH.joinpath("issue-fixes.md")),
)
def test_issue_fixes(line, title, input, expected):
    md = MarkdownIt()
    text = md.render(input)
    print(text)
    assert text.rstrip() == expected.rstrip()



from markdown_it import MarkdownIt, presets


def test_highlight_arguments():
    def highlight_func(str_, lang, attrs):
        assert lang == "a"
        assert attrs == "b  c  d"
        return "<pre><code>==" + str_ + "==</code></pre>"

    conf = presets.commonmark.make()
    conf["options"]["highlight"] = highlight_func
    md = MarkdownIt(config=conf)
    assert md.render("``` a  b  c  d \nhl\n```") == "<pre><code>==hl\n==</code></pre>\n"


def test_ordered_list_info():
    def type_filter(tokens, type_):
        return [t for t in tokens if t.type == type_]

    md = MarkdownIt()

    tokens = md.parse("1. Foo\n2. Bar\n20. Fuzz")
    assert len(type_filter(tokens, "ordered_list_open")) == 1
    tokens = type_filter(tokens, "list_item_open")
    assert len(tokens) == 3
    assert tokens[0].info == "1"
    assert tokens[0].markup == "."
    assert tokens[1].info == "2"
    assert tokens[1].markup == "."
    assert tokens[2].info == "20"
    assert tokens[2].markup == "."

    tokens = md.parse(" 1. Foo\n2. Bar\n  20. Fuzz\n 199. Flp")
    assert len(type_filter(tokens, "ordered_list_open")) == 1
    tokens = type_filter(tokens, "list_item_open")
    assert len(tokens) == 4
    assert tokens[0].info == "1"
    assert tokens[0].markup == "."
    assert tokens[1].info == "2"
    assert tokens[1].markup == "."
    assert tokens[2].info == "20"
    assert tokens[2].markup == "."
    assert tokens[3].info == "199"
    assert tokens[3].markup == "."



from markdown_it import MarkdownIt
from markdown_it.token import Token


def test_get_rules():
    md = MarkdownIt("zero")
    # print(md.get_all_rules())
    assert md.get_all_rules() == {
        "core": [
            "normalize",
            "block",
            "inline",
            "linkify",
            "replacements",
            "smartquotes",
            "text_join",
        ],
        "block": [
            "table",
            "code",
            "fence",
            "blockquote",
            "hr",
            "list",
            "reference",
            "html_block",
            "heading",
            "lheading",
            "paragraph",
        ],
        "inline": [
            "text",
            "linkify",
            "newline",
            "escape",
            "backticks",
            "strikethrough",
            "emphasis",
            "link",
            "image",
            "autolink",
            "html_inline",
            "entity",
        ],
        "inline2": ["balance_pairs", "strikethrough", "emphasis", "fragments_join"],
    }


def test_load_presets():
    md = MarkdownIt("zero")
    assert md.get_active_rules() == {
        "block": ["paragraph"],
        "core": ["normalize", "block", "inline", "text_join"],
        "inline": ["text"],
        "inline2": ["balance_pairs", "fragments_join"],
    }
    md = MarkdownIt("commonmark")
    assert md.get_active_rules() == {
        "core": ["normalize", "block", "inline", "text_join"],
        "block": [
            "code",
            "fence",
            "blockquote",
            "hr",
            "list",
            "reference",
            "html_block",
            "heading",
            "lheading",
            "paragraph",
        ],
        "inline": [
            "text",
            "newline",
            "escape",
            "backticks",
            "emphasis",
            "link",
            "image",
            "autolink",
            "html_inline",
            "entity",
        ],
        "inline2": ["balance_pairs", "emphasis", "fragments_join"],
    }


def test_override_options():
    md = MarkdownIt("zero")
    assert md.options["maxNesting"] == 20
    md = MarkdownIt("zero", {"maxNesting": 99})
    assert md.options["maxNesting"] == 99


def test_gfm_like2_tasklists_editable():
    md_default = MarkdownIt("gfm-like2")
    assert md_default.options["tasklists_editable"] is False
    assert (
        '<input class="task-list-item-checkbox" disabled="" type="checkbox" checked="">'
        in md_default.render("- [x] done")
    )

    md_editable = MarkdownIt("gfm-like2", {"tasklists_editable": True})
    assert md_editable.options["tasklists_editable"] is True
    assert (
        '<input class="task-list-item-checkbox" type="checkbox" checked="">'
        in md_editable.render("- [x] done")
    )


def test_gfm_like2_alert_token_map():
    md = MarkdownIt("gfm-like2")
    tokens = md.parse("> [!NOTE]\n> body")
    assert tokens[0].type == "alert_open"
    assert tokens[0].map == [0, 2]


def test_enable():
    md = MarkdownIt("zero").enable("heading")
    assert md.get_active_rules() == {
        "block": ["heading", "paragraph"],
        "core": ["normalize", "block", "inline", "text_join"],
        "inline": ["text"],
        "inline2": ["balance_pairs", "fragments_join"],
    }
    md.enable(["backticks", "autolink"])
    assert md.get_active_rules() == {
        "block": ["heading", "paragraph"],
        "core": ["normalize", "block", "inline", "text_join"],
        "inline": ["text", "backticks", "autolink"],
        "inline2": ["balance_pairs", "fragments_join"],
    }


def test_disable():
    md = MarkdownIt("zero").disable("inline")
    assert md.get_active_rules() == {
        "block": ["paragraph"],
        "core": ["normalize", "block", "text_join"],
        "inline": ["text"],
        "inline2": ["balance_pairs", "fragments_join"],
    }
    md.disable(["text"])
    assert md.get_active_rules() == {
        "block": ["paragraph"],
        "core": ["normalize", "block", "text_join"],
        "inline": [],
        "inline2": ["balance_pairs", "fragments_join"],
    }


def test_reset():
    md = MarkdownIt("zero")
    with md.reset_rules():
        md.disable("inline")
        assert md.get_active_rules() == {
            "block": ["paragraph"],
            "core": ["normalize", "block", "text_join"],
            "inline": ["text"],
            "inline2": ["balance_pairs", "fragments_join"],
        }
    assert md.get_active_rules() == {
        "block": ["paragraph"],
        "core": ["normalize", "block", "inline", "text_join"],
        "inline": ["text"],
        "inline2": ["balance_pairs", "fragments_join"],
    }


def test_parseInline():
    md = MarkdownIt()
    tokens = md.parseInline("abc\n\n> xyz")
    assert tokens == [
        Token(
            type="inline",
            tag="",
            nesting=0,
            attrs={},
            map=[0, 1],
            level=0,
            children=[
                Token(
                    type="text",
                    tag="",
                    nesting=0,
                    attrs={},
                    map=None,
                    level=0,
                    children=None,
                    content="abc",
                    markup="",
                    info="",
                    meta={},
                    block=False,
                    hidden=False,
                ),
                Token(
                    type="softbreak",
                    tag="br",
                    nesting=0,
                    attrs={},
                    map=None,
                    level=0,
                    children=None,
                    content="",
                    markup="",
                    info="",
                    meta={},
                    block=False,
                    hidden=False,
                ),
                Token(
                    type="softbreak",
                    tag="br",
                    nesting=0,
                    attrs={},
                    map=None,
                    level=0,
                    children=None,
                    content="",
                    markup="",
                    info="",
                    meta={},
                    block=False,
                    hidden=False,
                ),
                Token(
                    type="text",
                    tag="",
                    nesting=0,
                    attrs={},
                    map=None,
                    level=0,
                    children=None,
                    content="> xyz",
                    markup="",
                    info="",
                    meta={},
                    block=False,
                    hidden=False,
                ),
            ],
            content="abc\n\n> xyz",
            markup="",
            info="",
            meta={},
            block=False,
            hidden=False,
        )
    ]


def test_renderInline():
    md = MarkdownIt("zero")
    tokens = md.renderInline("abc\n\n*xyz*")
    assert tokens == "abc\n\n*xyz*"


def test_emptyStr():
    md = MarkdownIt()
    tokens = md.parseInline("")
    assert tokens == [
        Token(
            type="inline",
            tag="",
            nesting=0,
            attrs={},
            map=[0, 1],
            level=0,
            children=[],
            content="",
            markup="",
            info="",
            meta={},
            block=False,
            hidden=False,
        )
    ]


def test_empty_env():
    """Test that an empty `env` is mutated, not copied and mutated."""
    md = MarkdownIt()

    env = {}  # type: ignore
    md.render("[foo]: /url\n[foo]", env)
    assert "references" in env

    env = {}
    md.parse("[foo]: /url\n[foo]", env)
    assert "references" in env


def test_table_tokens(data_regression):
    md = MarkdownIt("js-default")
    tokens = md.parse(
        """
| Heading 1 | Heading 2
| --------- | ---------
| Cell 1    | Cell 2
| Cell 3    | Cell 4
    """
    )
    data_regression.check([t.as_dict() for t in tokens])


def test_fragments_join_merges_adjacent_text_tokens():
    """fragments_join should merge runs of adjacent text tokens into one.

    Underscore characters flanked by word characters (e.g. ``a_b``) are not
    valid emphasis delimiters in CommonMark, so the emphasis rule leaves each
    ``_`` as a plain text token adjacent to the surrounding text tokens,
    giving a run of five tokens: text("a"), text("_"), text("b c"),
    text("_"), text("d").

    Note: there is also a core-level ``text_join`` rule that collapses adjacent
    text tokens as a fallback.  We disable it here so that the assertions are
    sensitive only to ``fragments_join``.
    """
    src = "a_b c_d"

    # --- both rules disabled: five separate text tokens must survive ---
    md_both_off = MarkdownIt()
    md_both_off.disable(["text_join", "fragments_join"])
    children_both_off = md_both_off.parseInline(src)[0].children
    assert children_both_off is not None
    assert len(children_both_off) > 1, "expected multiple text tokens with no merging"
    assert all(t.type == "text" for t in children_both_off)

    # --- only fragments_join enabled (text_join still off): run must collapse ---
    md_fj_on = MarkdownIt()
    md_fj_on.disable("text_join")
    children_fj_on = md_fj_on.parseInline(src)[0].children
    assert children_fj_on is not None
    assert len(children_fj_on) == 1
    assert children_fj_on[0].type == "text"
    assert children_fj_on[0].content == "a_b c_d"


def test_text_join_merges_adjacent_text_special_tokens():
    """text_join should convert text_special tokens and merge runs into one.

    Backslash-escaped characters each produce a ``text_special`` token.
    ``fragments_join`` only merges ``text`` tokens, so a run of
    ``text_special`` tokens passes through it untouched.  ``text_join``
    must then convert them to ``text`` and collapse the run in a single
    pass rather than via pairwise concatenation.
    """
    # Three consecutive backslash escapes → three text_special tokens before
    # text_join runs.
    src = r"\*\*\*"

    # --- text_join disabled: three text_special tokens must survive ---
    md_off = MarkdownIt()
    md_off.disable("text_join")
    children_off = md_off.parseInline(src)[0].children
    assert children_off is not None
    assert len(children_off) > 1, "expected multiple text_special tokens before merging"
    assert all(t.type == "text_special" for t in children_off)

    # --- text_join enabled (default): must collapse to a single text token ---
    md_on = MarkdownIt()
    children_on = md_on.parseInline(src)[0].children
    assert children_on is not None
    assert len(children_on) == 1
    assert children_on[0].type == "text"
    assert children_on[0].content == "***"



def pytest_benchmark_update_machine_info(config, machine_info):
    import psutil

    freq = psutil.cpu_freq()
    machine_info["psutil"] = {
        "cpu_count": psutil.cpu_count(logical=False),
        "cpu_count_logical": psutil.cpu_count(logical=True),
        "cpu_percent": psutil.cpu_percent(),
        "cpu_freq_min": freq.min,
        "cpu_freq_max": freq.max,
        "cpu_freq_current": freq.current,
    }



from markdown_it import MarkdownIt
from markdown_it.tree import SyntaxTreeNode

EXAMPLE_MARKDOWN = """
## Heading here

Some paragraph text and **emphasis here** and more text here.
"""


def test_tree_to_tokens_conversion():
    tokens = MarkdownIt().parse(EXAMPLE_MARKDOWN)
    tokens_after_roundtrip = SyntaxTreeNode(tokens).to_tokens()
    assert tokens == tokens_after_roundtrip


def test_property_passthrough():
    tokens = MarkdownIt().parse(EXAMPLE_MARKDOWN)
    heading_open = tokens[0]
    tree = SyntaxTreeNode(tokens)
    heading_node = tree.children[0]
    assert heading_open.tag == heading_node.tag
    assert tuple(heading_open.map or ()) == heading_node.map
    assert heading_open.level == heading_node.level
    assert heading_open.content == heading_node.content
    assert heading_open.markup == heading_node.markup
    assert heading_open.info == heading_node.info
    assert heading_open.meta == heading_node.meta
    assert heading_open.block == heading_node.block
    assert heading_open.hidden == heading_node.hidden


def test_type():
    tokens = MarkdownIt().parse(EXAMPLE_MARKDOWN)
    tree = SyntaxTreeNode(tokens)
    # Root type is "root"
    assert tree.type == "root"
    # "_open" suffix must be stripped from nested token type
    assert tree.children[0].type == "heading"
    assert tree[0].type == "heading"
    # For unnested tokens, node type must remain same as token type
    assert tree.children[0].children[0].type == "inline"


def test_sibling_traverse():
    tokens = MarkdownIt().parse(EXAMPLE_MARKDOWN)
    tree = SyntaxTreeNode(tokens)
    paragraph_inline_node = tree.children[1].children[0]
    text_node = paragraph_inline_node.children[0]
    assert text_node.type == "text"
    strong_node = text_node.next_sibling
    assert strong_node
    assert strong_node.type == "strong"
    another_text_node = strong_node.next_sibling
    assert another_text_node
    assert another_text_node.type == "text"
    assert another_text_node.next_sibling is None
    assert another_text_node.previous_sibling.previous_sibling == text_node  # type: ignore
    assert text_node.previous_sibling is None


def test_pretty(file_regression):
    md = MarkdownIt("commonmark")
    tokens = md.parse(
        """
# Header

Here's some text and an image ![title](image.png)

1. a **list**

> a *quote*
    """
    )
    node = SyntaxTreeNode(tokens)
    file_regression.check(node.pretty(indent=2, show_text=True), extension=".xml")


def test_pretty_text_special(file_regression):
    md = MarkdownIt()
    md.disable("text_join")
    tree = SyntaxTreeNode(md.parse("foo &copy; bar \\("))
    file_regression.check(tree.pretty(show_text=True), extension=".xml")


def test_walk():
    tokens = MarkdownIt().parse(EXAMPLE_MARKDOWN)
    tree = SyntaxTreeNode(tokens)
    expected_node_types = (
        "root",
        "heading",
        "inline",
        "text",
        "paragraph",
        "inline",
        "text",
        "strong",
        "text",
        "text",
    )
    for node, expected_type in zip(tree.walk(), expected_node_types):
        assert node.type == expected_type



import sys

import atheris

# Beautified from auto-generated fuzzer at:
# https://github.com/ossf/fuzz-introspector/pull/872#issuecomment-1450847118
# Auto-fuzz heuristics used: py-autofuzz-heuristics-4.1
# Imports by the generated code
import markdown_it


def TestOneInput(data):
    fdp = atheris.FuzzedDataProvider(data)
    val_1 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 1024))
    val_2 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 1024))
    val_3 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_4 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_5 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_6 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_7 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_8 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_9 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))
    val_10 = fdp.ConsumeUnicodeNoSurrogates(fdp.ConsumeIntInRange(0, 256))

    try:
        c1 = markdown_it.main.MarkdownIt()
        c1.render(val_1)
        c1.parse(val_2)
        c1.renderInline(val_3)
        c1.parseInline(val_4)
        c1.normalizeLink(val_5)
        c1.normalizeLinkText(val_6)
        c1.disable(val_7)
        c1.enable(val_8)
        c1.validateLink(val_9)
        c1.configure(val_10)
    except (
        ValueError,
        KeyError,
        TypeError,
    ):
        # Exceptions thrown by the hit code.
        pass


def main():
    atheris.instrument_all()
    atheris.Setup(sys.argv, TestOneInput)
    atheris.Fuzz()


if __name__ == "__main__":
    main()



# Skip text characters for text token, place those to pending buffer
# and increment current pos
from .state_inline import StateInline

# Rule to skip pure text


def text(state: StateInline, silent: bool) -> bool:
    pos = state.pos
    posMax = state.posMax

    terminator_char = state.md.inline.terminator_re.search(state.src, pos)
    pos = terminator_char.start() if terminator_char else posMax

    if pos == state.pos:
        return False

    if not silent:
        state.pending += state.src[state.pos : pos]

    state.pos = pos

    return True



from markdown_it import MarkdownIt


def test_token_levels():
    mdit = MarkdownIt(options_update={"linkify": True}).enable("linkify")
    tokens = mdit.parse("www.python.org")
    inline = tokens[1]
    assert inline.type == "inline"
    assert inline.children
    link_open = inline.children[0]
    assert link_open.type == "link_open"
    link_text = inline.children[1]
    assert link_text.type == "text"
    link_close = inline.children[2]
    assert link_close.type == "link_close"

    # Assert that linkify tokens have correct nesting levels
    assert link_open.level == 0
    assert link_text.level == 1
    assert link_close.level == 0



import sys

import atheris

from markdown_it import MarkdownIt


def TestOneInput(data):
    fdp = atheris.FuzzedDataProvider(data)
    md = MarkdownIt()
    raw_markdown = fdp.ConsumeUnicodeNoSurrogates(sys.maxsize)
    md.parse(raw_markdown)
    md.render(raw_markdown)


def main():
    atheris.instrument_all()
    atheris.Setup(sys.argv, TestOneInput)
    atheris.Fuzz()


if __name__ == "__main__":
    main()



"""A script for profiling.

To generate and read results:
  - `tox -e profile`
  - `firefox .tox/prof/output.svg`
"""

from pathlib import Path

from markdown_it import MarkdownIt

commonmark_spec = (
    (Path(__file__).parent.parent / "tests" / "test_cmark_spec" / "spec.md")
    .read_bytes()
    .decode()
)

# Run this a few times to emphasize over imports and other overhead above
for _ in range(10):
    MarkdownIt().render(commonmark_spec)



from contextlib import redirect_stdout
import io
import pathlib
import tempfile
from unittest.mock import patch

import pytest

from markdown_it.cli import parse


def test_parse():
    with tempfile.TemporaryDirectory() as tempdir:
        path = pathlib.Path(tempdir).joinpath("test.md")
        path.write_text("a b c")
        assert parse.main([str(path)]) == 0


def test_parse_fail():
    with pytest.raises(SystemExit) as exc_info:
        parse.main(["/tmp/nonexistant_path/for_cli_test.md"])
    assert exc_info.value.code == 1


def test_non_utf8():
    with tempfile.TemporaryDirectory() as tempdir:
        path = pathlib.Path(tempdir).joinpath("test.md")
        path.write_bytes(b"\x80abc")
        assert parse.main([str(path)]) == 0


def test_print_heading():
    with patch("builtins.print") as patched:
        parse.print_heading()
    patched.assert_called()


def test_interactive():
    def mock_input(prompt):
        raise KeyboardInterrupt

    with patch("builtins.print") as patched, patch("builtins.input", mock_input):
        parse.interactive()
    patched.assert_called()


def test_main_no_args_is_interactive():
    with patch("markdown_it.cli.parse.interactive") as mock_interactive:
        assert parse.main([]) == 0
    mock_interactive.assert_called_once()


def test_parse_output():
    with tempfile.TemporaryDirectory() as tempdir:
        path = pathlib.Path(tempdir).joinpath("test.md")
        path.write_text("# a b c")
        string_io = io.StringIO()
        with redirect_stdout(string_io):
            assert parse.main([str(path)]) == 0
        assert string_io.getvalue() == "<h1>a b c</h1>\n"


def test_stdin():
    with patch("sys.stdin", io.StringIO("# a b c")):
        string_io = io.StringIO()
        with redirect_stdout(string_io):
            assert parse.main(["--stdin"]) == 0
        assert string_io.getvalue() == "<h1>a b c</h1>\n"


def test_multiple_files():
    with tempfile.TemporaryDirectory() as tempdir:
        path1 = pathlib.Path(tempdir).joinpath("test1.md")
        path1.write_text("# file 1")
        path2 = pathlib.Path(tempdir).joinpath("test2.md")
        path2.write_text("* file 2")
        string_io = io.StringIO()
        with redirect_stdout(string_io):
            assert parse.main([str(path1), str(path2)]) == 0
        assert string_io.getvalue() == "<h1>file 1</h1>\n<ul>\n<li>file 2</li>\n</ul>\n"


def test_interactive_render():
    # Simulate user typing '# hello', pressing Ctrl-D (renders), then Ctrl-C (exits)
    # This is needed to break the infinite loop in interactive mode on EOF.
    mock_input = patch(
        "builtins.input", side_effect=["# hello", EOFError, KeyboardInterrupt]
    )
    string_io = io.StringIO()
    with redirect_stdout(string_io), mock_input:
        parse.interactive()

    output = string_io.getvalue()
    assert "markdown-it-py" in output  # from print_heading
    # The rendered output is prefixed by a newline
    assert "\n<h1>hello</h1>\n" in output
    assert "Exiting" in output



"""
These tests are in response to reports from:
https://github.com/google/oss-fuzz/tree/master/projects/markdown-it-py

In the future, perhaps atheris could be directly used here,
but it was not directly apparent how to integrate it into pytest.
"""

import pytest

from markdown_it import MarkdownIt

TESTS = {
    55363: (">```\n>", "<blockquote>\n<pre><code></code></pre>\n</blockquote>\n"),
    55367: (">-\n>\n>", "<blockquote>\n<ul>\n<li></li>\n</ul>\n</blockquote>\n"),
    55371: ("[](so&#4H0;!", "<p>[](so&amp;#4H0;!</p>\n"),
    # 55401: (("?c_" * 100000) + "c_", ""),  TODO this does not fail, just takes a long time
}


@pytest.mark.parametrize("raw_input,expected", TESTS.values(), ids=TESTS.keys())
def test_fuzzing(raw_input, expected):
    md = MarkdownIt()
    md.parse(raw_input)
    assert md.render(raw_input) == expected



from pathlib import Path

import pytest

import markdown_it


@pytest.fixture
def spec_text():
    return Path(__file__).parent.joinpath("samples", "spec.md").read_text()


@pytest.fixture
def parser():
    return markdown_it.MarkdownIt("commonmark")


@pytest.mark.benchmark(group="core")
def test_spec(benchmark, parser, spec_text):
    benchmark(parser.render, spec_text)



"""Test basic plugin creation functionality:
that they can be added and are called correctly
"""

from markdown_it import MarkdownIt


def inline_rule(state, silent):
    print("plugin called")
    return False


def test_inline_after(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.inline.ruler.after("text", "new_rule", inline_rule)

    MarkdownIt().use(_plugin).parse("[")
    assert "plugin called" in capsys.readouterr().out


def test_inline_before(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.inline.ruler.before("text", "new_rule", inline_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_inline_at(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.inline.ruler.at("text", inline_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def block_rule(state, startLine, endLine, silent):
    print("plugin called")
    return False


def test_block_after(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.block.ruler.after("hr", "new_rule", block_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_block_before(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.block.ruler.before("hr", "new_rule", block_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_block_at(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.block.ruler.at("hr", block_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def core_rule(state):
    print("plugin called")


def test_core_after(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.core.ruler.after("normalize", "new_rule", core_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_core_before(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.core.ruler.before("normalize", "new_rule", core_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_core_at(capsys):
    def _plugin(_md: MarkdownIt) -> None:
        _md.core.ruler.at("normalize", core_rule)

    MarkdownIt().use(_plugin).parse("a")
    assert "plugin called" in capsys.readouterr().out


def test_add_terminator_char():
    """Test that add_terminator_char stops the text rule on a new character."""
    hit_positions = []

    def w_rule(state, silent):
        if state.src[state.pos] != "w":
            return False
        hit_positions.append(state.pos)
        state.pos += 1
        return True

    def _plugin(_md: MarkdownIt) -> None:
        _md.inline.add_terminator_char("w")
        _md.inline.ruler.before("text", "w_rule", w_rule)

    md = MarkdownIt().use(_plugin)

    # Without the terminator 'w' would be consumed as plain text;
    # with it the rule fires exactly for the 'w' at position 1 in "awb".
    md.render("awb")
    assert hit_positions == [1]


def test_add_terminator_char_idempotent():
    """add_terminator_char with an already-present char should not rebuild the regex."""
    md = MarkdownIt()
    original_re = md.inline.terminator_re

    # '\n' is already in the default set – adding it again must not rebuild
    md.inline.add_terminator_char("\n")
    assert md.inline.terminator_re is original_re


def test_add_terminator_char_rebuilds():
    """add_terminator_char with a new char should rebuild the regex."""
    md = MarkdownIt()
    original_re = md.inline.terminator_re

    md.inline.add_terminator_char("w")
    assert md.inline.terminator_re is not original_re
    assert "w" in md.inline._extra_terminator_chars



from .state_inline import StateInline


def fragments_join(state: StateInline) -> None:
    """
    Clean up tokens after emphasis and strikethrough postprocessing:
    merge adjacent text nodes into one and re-calculate all token levels

    This is necessary because initially emphasis delimiter markers (``*, _, ~``)
    are treated as their own separate text tokens. Then emphasis rule either
    leaves them as text (needed to merge with adjacent text) or turns them
    into opening/closing tags (which messes up levels inside).
    """
    level = 0
    maximum = len(state.tokens)

    curr = last = 0
    while curr < maximum:
        # re-calculate levels after emphasis/strikethrough turns some text nodes
        # into opening/closing tags
        if state.tokens[curr].nesting < 0:
            level -= 1  # closing tag
        state.tokens[curr].level = level
        if state.tokens[curr].nesting > 0:
            level += 1  # opening tag

        if (
            state.tokens[curr].type == "text"
            and curr + 1 < maximum
            and state.tokens[curr + 1].type == "text"
        ):
            # Collapse a run of adjacent text nodes in a single join, instead
            # of pairwise `a + b` concatenation. The pairwise form is O(L*k)
            # in the size of the run because each step rebuilds the growing
            # prefix; "".join is O(L).
            parts = [state.tokens[curr].content]
            curr += 1
            while curr < maximum and state.tokens[curr].type == "text":
                parts.append(state.tokens[curr].content)
                curr += 1
            merged = state.tokens[curr - 1]
            merged.content = "".join(parts)
            merged.level = level
            state.tokens[last] = merged
            last += 1
            continue

        if curr != last:
            state.tokens[last] = state.tokens[curr]
        last += 1
        curr += 1

    if curr != last:
        del state.tokens[last:]



# /// script
# dependencies = [
#   "requests",
# ]
# ///
from pathlib import Path
from typing import Any

default_version = "0.31.2"
default_json_path = Path(__file__).parent / "commonmark.json"
default_text_path = Path(__file__).parent / "spec.md"
default_fixture_path = (
    Path(__file__).parent.parent / "test_port" / "fixtures" / "commonmark_spec.md"
)


def create_argparser():
    import argparse

    parser = argparse.ArgumentParser(description="Download CommonMark spec JSON")
    parser.add_argument(
        "version",
        nargs="?",
        default=default_version,
        help=f"CommonMark spec version to download (default: {default_version})",
    )
    parser.add_argument(
        "--output-json",
        type=Path,
        default=default_json_path,
        help=f"Output file path (default: {default_json_path})",
    )
    parser.add_argument(
        "--output-text",
        type=Path,
        default=default_text_path,
        help=f"Output file path (default: {default_text_path})",
    )
    parser.add_argument(
        "--output-fixture",
        type=Path,
        default=default_fixture_path,
        help=f"Write to test fixture (default: {default_fixture_path})",
    )
    return parser


def _json_to_fixture(data: list[dict[str, Any]]) -> str:
    text = ""
    for item in data:
        text += "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
        text += f"src line: {item['start_line'] - 1}\n\n"
        text += f".\n{item['markdown']}.\n{item['html']}.\n\n"
    return text


if __name__ == "__main__":
    import requests  # type: ignore[import-untyped]

    args = create_argparser().parse_args()
    version: str = args.version
    json_path: Path = args.output_json
    txt_path: Path = args.output_text
    test_fixture: Path = args.output_fixture

    changed = False

    json_url = f"https://spec.commonmark.org/{version}/spec.json"
    txt_url = f"https://raw.githubusercontent.com/commonmark/commonmark-spec/refs/tags/{version}/spec.txt"

    for url, output_path in ((json_url, json_path), (txt_url, txt_path)):
        print(f"Downloading CommonMark spec from {url}")
        response = requests.get(url)
        response.raise_for_status()
        if not output_path.exists() or output_path.read_text() != response.text:
            changed = True
            with output_path.open("w") as f:
                f.write(response.text)
            print(f"Updated to {output_path}")
        else:
            print(f"File {output_path} is up to date, not overwriting")

    # write_to_test_fixture:
    response = requests.get(json_url)
    response.raise_for_status()
    data = response.json()
    text = _json_to_fixture(data)
    if not test_fixture.exists() or test_fixture.read_text() != text:
        changed = True
        with test_fixture.open("w") as f:
            f.write(text)
        print(f"Also updated to {test_fixture}")
    else:
        print(f"Fixture file {test_fixture} is up to date, not overwriting")

    raise SystemExit(0 if not changed else 1)



__all__ = ("commonmark", "default", "gfm_like", "gfm_like2", "js_default", "zero")

from ..utils import PresetType
from . import commonmark, default, zero

js_default = default


class gfm_like:  # noqa: N801
    """GitHub Flavoured Markdown (GFM) like.

    This adds the linkify, table and strikethrough components to CommmonMark.

    Note, it lacks task-list items and raw HTML filtering,
    to meet the the full GFM specification
    (see https://github.github.com/gfm/#autolinks-extension-).
    """

    @staticmethod
    def make() -> PresetType:
        config = commonmark.make()
        config["components"]["core"]["rules"].append("linkify")
        config["components"]["block"]["rules"].append("table")
        config["components"]["inline"]["rules"].extend(["strikethrough", "linkify"])
        config["components"]["inline"]["rules2"].append("strikethrough")
        config["options"]["linkify"] = True
        config["options"]["html"] = True
        return config


class gfm_like2:  # noqa: N801
    """GitHub Flavoured Markdown (GFM) like, extended.

    Builds on ``gfm-like`` and additionally enables:

    - Task lists (``- [x] done``)
    - Alerts (``> [!NOTE]``)
    - Single-tilde strikethrough (``~text~`` in addition to ``~~text~~``)
    """

    @staticmethod
    def make() -> PresetType:
        config = gfm_like.make()
        config["options"]["tasklists"] = True
        config["options"]["tasklists_editable"] = False
        config["options"]["alerts"] = True
        config["options"]["strikethrough_single_tilde"] = True
        return config



__all__ = (
    "StateBlock",
    "blockquote",
    "code",
    "fence",
    "heading",
    "hr",
    "html_block",
    "lheading",
    "list_block",
    "make_fence_rule",
    "paragraph",
    "reference",
    "table",
)

from .blockquote import blockquote
from .code import code
from .fence import fence, make_fence_rule
from .heading import heading
from .hr import hr
from .html_block import html_block
from .lheading import lheading
from .list import list_block
from .paragraph import paragraph
from .reference import reference
from .state_block import StateBlock
from .table import table



"""Tests for make_fence_rule factory function."""

from __future__ import annotations

from markdown_it import MarkdownIt
from markdown_it.rules_block.fence import make_fence_rule


def _make_colon_fence_md() -> MarkdownIt:
    """Create a MarkdownIt instance with a colon fence rule (like colon_fence plugin)."""
    md = MarkdownIt()
    colon_rule = make_fence_rule(
        markers=(":",),
        token_type="colon_fence",
        disallow_marker_in_info=(),
    )
    md.block.ruler.before(
        "fence",
        "colon_fence",
        colon_rule,
        {"alt": ["paragraph", "reference", "blockquote", "list"]},
    )
    return md


class TestColonFenceMarker:
    """Test colon fence behavior (`:` marker, like mdit-py-plugins colon_fence)."""

    def test_basic(self):
        md = _make_colon_fence_md()
        tokens = md.parse(":::\nfoo\n:::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert tokens[0].content == "foo\n"
        assert tokens[0].markup == ":::"

    def test_with_info(self):
        md = _make_colon_fence_md()
        tokens = md.parse(":::python\nfoo\n:::\n")
        assert tokens[0].type == "colon_fence"
        assert tokens[0].info == "python"
        assert tokens[0].content == "foo\n"

    def test_colon_in_info_allowed(self):
        """Colons in info string should be allowed (unlike backticks in backtick fences)."""
        md = _make_colon_fence_md()
        tokens = md.parse(":::{note}\nfoo\n:::\n")
        assert tokens[0].type == "colon_fence"
        assert tokens[0].info == "{note}"

    def test_longer_closing(self):
        """Closing fence can be longer than opening (default, not exact_match)."""
        md = _make_colon_fence_md()
        tokens = md.parse(":::\nfoo\n::::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert tokens[0].content == "foo\n"

    def test_shorter_closing_no_match(self):
        """Closing fence shorter than opening does not close the block."""
        md = _make_colon_fence_md()
        tokens = md.parse("::::\nfoo\n:::\nbar\n::::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        # The inner ::: is content, not a closing fence
        assert ":::" in tokens[0].content
        assert "foo" in tokens[0].content
        assert "bar" in tokens[0].content

    def test_does_not_interfere_with_backtick(self):
        """Standard backtick fences still work alongside colon fences."""
        md = _make_colon_fence_md()
        tokens = md.parse("```\nfoo\n```\n")
        assert tokens[0].type == "fence"
        assert tokens[0].content == "foo\n"

    def test_unclosed_block(self):
        """Unclosed colon fence is autoclosed at end of document."""
        md = _make_colon_fence_md()
        tokens = md.parse(":::\nfoo\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert tokens[0].content == "foo\n"


class TestExactMatch:
    """Test exact_match option for fence closing."""

    def _make_exact_md(self) -> MarkdownIt:
        """Create MarkdownIt with exact-match colon fence."""
        md = MarkdownIt()
        colon_rule = make_fence_rule(
            markers=(":",),
            token_type="colon_fence",
            disallow_marker_in_info=(),
            exact_match=True,
        )
        md.block.ruler.before(
            "fence",
            "colon_fence",
            colon_rule,
            {"alt": ["paragraph", "reference", "blockquote", "list"]},
        )
        return md

    def test_exact_match_same_length(self):
        """Closing fence with same length closes the block."""
        md = self._make_exact_md()
        tokens = md.parse(":::\nfoo\n:::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert tokens[0].content == "foo\n"

    def test_exact_match_longer_no_close(self):
        """Closing fence with more markers does NOT close (exact match required)."""
        md = self._make_exact_md()
        tokens = md.parse(":::\nfoo\n::::\n:::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        # :::: is content, not a closer; ::: at the end is the real closer
        assert "::::" in tokens[0].content
        assert tokens[0].content == "foo\n::::\n"

    def test_exact_match_shorter_no_close(self):
        """Closing fence with fewer markers does NOT close (exact match required)."""
        md = self._make_exact_md()
        tokens = md.parse("::::\nfoo\n:::\nbar\n::::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert ":::" in tokens[0].content
        assert tokens[0].content == "foo\n:::\nbar\n"

    def test_nesting_pattern(self):
        """Demonstrate nested fences with exact match (outer 4, inner 3)."""
        md = self._make_exact_md()
        tokens = md.parse(
            "::::{note}\nouter\n:::{warning}\ninner\n:::\nouter again\n::::\n"
        )
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        assert tokens[0].info == "{note}"
        assert tokens[0].markup == "::::"
        # The inner ::: fence is captured as content
        assert ":::{warning}" in tokens[0].content
        assert "inner" in tokens[0].content
        assert ":::" in tokens[0].content
        assert "outer again" in tokens[0].content

    def test_unclosed_exact_match(self):
        """Unclosed exact-match fence is autoclosed at end of document."""
        md = self._make_exact_md()
        tokens = md.parse(":::\nfoo\n::::\n")
        assert len(tokens) == 1
        assert tokens[0].type == "colon_fence"
        # :::: doesn't close (wrong length), so content includes it
        assert tokens[0].content == "foo\n::::\n"


class TestOverrideStandardFence:
    """Test overriding the standard fence rule with make_fence_rule."""

    def test_override_with_exact_match(self):
        """Override standard fence to use exact matching."""
        md = MarkdownIt()
        md.block.ruler.at("fence", make_fence_rule(exact_match=True))
        tokens = md.parse("```\nfoo\n````\n```\n")
        assert tokens[0].type == "fence"
        # ```` doesn't close (exact match), ``` does
        assert tokens[0].content == "foo\n````\n"

    def test_override_add_colon_marker(self):
        """Override standard fence to also accept colon markers."""
        md = MarkdownIt()
        md.block.ruler.at(
            "fence",
            make_fence_rule(markers=("~", "`", ":")),
        )
        tokens = md.parse(":::\nfoo\n:::\n")
        assert tokens[0].type == "fence"
        assert tokens[0].content == "foo\n"
        assert tokens[0].markup == ":::"

    def test_override_preserves_backtick_info_restriction(self):
        """Backtick info string restriction still works with custom markers."""
        md = MarkdownIt()
        md.block.ruler.at(
            "fence",
            make_fence_rule(markers=("~", "`", ":")),
        )
        # Backtick in info string of backtick fence → not a fence
        tokens = md.parse("``` foo`bar\ntest\n```\n")
        assert not any(t.type == "fence" for t in tokens)

        # Colon in info string of colon fence → allowed (not in disallow list)
        tokens = md.parse(":::foo:bar\ntest\n:::\n")
        assert tokens[0].type == "fence"
        assert tokens[0].info == "foo:bar"


class TestMinMarkers:
    """Test min_markers parameter."""

    def test_min_markers_default(self):
        """Default min_markers is 3."""
        md = MarkdownIt()
        tokens = md.parse("``\nfoo\n``\n")
        # 2 backticks is not a fence
        assert not any(t.type == "fence" for t in tokens)

    def test_min_markers_custom(self):
        """Custom min_markers allows changing the threshold."""
        md = MarkdownIt()
        md.block.ruler.at("fence", make_fence_rule(min_markers=4))
        # 3 backticks is now below threshold
        tokens = md.parse("```\nfoo\n```\n")
        assert not any(t.type == "fence" for t in tokens)
        # 4 backticks works
        tokens = md.parse("````\nfoo\n````\n")
        assert tokens[0].type == "fence"
        assert tokens[0].content == "foo\n"


class TestDisallowMarkerInInfo:
    """Test disallow_marker_in_info parameter."""

    def test_default_backtick_disallowed(self):
        """By default, backtick in backtick-fence info string is rejected."""
        md = MarkdownIt()
        tokens = md.parse("```foo`bar\ntest\n```\n")
        # Not parsed as a fence (`` ` `` in info rejects it as opener)
        assert not any(t.type == "fence" and "foo`bar" in t.info for t in tokens)

    def test_tilde_in_tilde_info_allowed(self):
        """By default, tilde in tilde-fence info string is allowed."""
        md = MarkdownIt()
        tokens = md.parse("~~~ foo~bar\ntest\n~~~\n")
        assert tokens[0].type == "fence"
        assert tokens[0].info == " foo~bar"

    def test_disallow_all(self):
        """Can disallow marker in info for all marker types."""
        md = MarkdownIt()
        md.block.ruler.at(
            "fence",
            make_fence_rule(disallow_marker_in_info=("~", "`")),
        )
        tokens = md.parse("~~~ foo~bar\ntest\n~~~\n")
        # Tilde in info string of tilde fence → rejected
        assert not any(t.type == "fence" for t in tokens)

    def test_disallow_none(self):
        """Can allow marker in info for all marker types."""
        md = MarkdownIt()
        md.block.ruler.at(
            "fence",
            make_fence_rule(disallow_marker_in_info=()),
        )
        # Backtick in backtick-fence info string → now allowed
        tokens = md.parse("``` foo`bar\ntest\n```\n")
        assert tokens[0].type == "fence"
        assert tokens[0].info == " foo`bar"
